using Microsoft.AspNetCore.Components;

using NSeguin.Dev.Web.Common.Data;

using static NSeguin.Dev.Web.LocalStorageAccessor;

namespace NSeguin.Dev.Web;

public class AppStateService
{
    private readonly LocalStorageAccessor _localStorageAccessor;

    public AppStateService(LocalStorageAccessor localStorageAccessor)
    {
        _localStorageAccessor = localStorageAccessor
                                ?? throw new ArgumentNullException(nameof(localStorageAccessor));

        AppState = new AsyncLocal<PersistedAppState>();
    }

    public AsyncLocal<PersistedAppState> AppState { get; }

    public bool IsInitialized { get; private set; }

    public async Task InitializeAsync()
    {
        _ = await GetAppStateAsync();
        IsInitialized = true;
    }

    public async ValueTask<PersistedAppState> GetAppStateAsync()
    {
        AppState.Value ??= await GetAppStateFromLocalStorageAsync();
        return AppState.Value;
    }

    public async Task SaveAppStateAsync()
    {
        if (AppState.Value == null)
        {
            return;
        }

        await SaveAppStateToLocalStorageAsync(AppState.Value);
    }

    private async ValueTask<PersistedAppState> GetAppStateFromLocalStorageAsync()
    {
        var persistedAppState
            = await _localStorageAccessor.GetValueAsync<PersistedAppState>(
                LocalStorageKeys.AppState);

        return persistedAppState;
    }

    private async ValueTask SaveAppStateToLocalStorageAsync(PersistedAppState persistedAppState)
    {
        await _localStorageAccessor.SetValueAsync(LocalStorageKeys.AppState, persistedAppState);
    }
}
