using Microsoft.JSInterop;

namespace NSeguin.Dev.Web;

public class LocalStorageAccessor : IAsyncDisposable
{
    public static class LocalStorageKeys
    {
        public const string AppState = "AppState";
    }

    private readonly IJSRuntime _jsRuntime;
    private IJSObjectReference? _jsModule;

    public LocalStorageAccessor(IJSRuntime jsRuntime)
    {
        _jsRuntime = jsRuntime;
    }

    public async Task InitializeAsync()
    {
        _jsModule ??= await _jsRuntime.InvokeAsync<IJSObjectReference>(
            "import",
            "./js/LocalStorageAccessor.js");
    }

    public async Task<T> GetValueAsync<T>(string key)
    {
        await InitializeAsync();
        var result = await _jsModule!.InvokeAsync<T>("get", key);
        return result;
    }

    public async Task SetValueAsync<T>(string key, T value)
    {
        await InitializeAsync();
        await _jsModule!.InvokeVoidAsync("set", key, value);
    }

    public async Task Clear()
    {
        await InitializeAsync();
        await _jsModule!.InvokeVoidAsync("clear");
    }

    public async Task RemoveAsync(string key)
    {
        await InitializeAsync();
        await _jsModule!.InvokeVoidAsync("remove", key);
    }

    public async ValueTask DisposeAsync()
    {
        if (_jsModule != null)
        {
            await _jsModule.DisposeAsync();
        }
    }
}
